<?php

namespace Drp\LdapAuthentication;

use Adldap\Laravel\AdldapAuthUserProvider;
use Adldap\Models\User;
use Drp\LdapAuthentication\Events\LdapUserGroupEvent;
use Illuminate\Contracts\Hashing\Hasher;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Arr;
/**
 * Class DrpldapAuthUserProvider
 * @package Drp\LdapAuthentication
 */
class DrpldapAuthUserProvider extends AdldapAuthUserProvider
{
    /**
     * Class for finder the best group for a user.
     * @var GroupFinder
     */
    protected $groupFinder;

    /**
     * Chosen group of the ldap user.
     * @var string
     */
    protected $chosenGroup;

    /**
     * Create a new database user provider.
     * @param Hasher $hasher Hashing alogirthm to use.
     * @param string $model Model of the user to auth.
     * @param GroupFinder $groupFinder A group finder instance.
     */
    public function __construct(Hasher $hasher, $model, GroupFinder $groupFinder)
    {
        $this->model = $model;
        $this->hasher = $hasher;
        $this->groupFinder = $groupFinder;
    }

    /**
     * Creates a local User from Active Directory.
     * @param User $user The Adldap user to convert.
     * @param string $password The password used to login.
     * @return \Illuminate\Database\Eloquent\Model|null
     */
    protected function getModelFromAdldap(User $user, $password)
    {
        // Check if the user is in the right group before building the model
        if (!$this->checkLDAPGroups($user)) {
            return null;
        }

        // Get the username attributes.
        $attributes = $this->getUsernameAttribute();

        // Get the model key.
        $key = key($attributes);

        // Get the username from the AD model.
        $username = $user->{$attributes[$key]};

        // Make sure we retrieve the first username
        // result if it's an array.
        if (is_array($username)) {
            $username = Arr::get($username, 0);
        }

        // Try to retrieve the model from the model key and AD username.
        //Include any thrashed models as they may now have group access enabled
        $model = $this->createModel()->newQuery()->withTrashed()->where([$key => $username])->first();

        // Create the model instance of it isn't found.
        if (!$model instanceof Model) {
            $model = $this->createModel();
        }

        // Set the username and password in case
        // of changes in active directory.
        $model->{$key} = $username;

        // Sync the users password.
        $model = $this->syncModelPassword($model, $password);

        // Synchronize other active directory
        // attributes on the model.
        $model = $this->syncModelFromAdldap($user, $model);

        if ($this->getBindUserToModel()) {
            $model = $this->bindAdldapToModel($user, $model);
        }


        //If the user is currently deleted it should be restored
        //This is because they have group access
        if ($user->trashed()) {
            $user->explicit_access = false;
            $user->restore();
        }

        event(new LdapUserGroupEvent($this->chosenGroup, $model));

        return $model;
    }

    /**
     * Check if the specified user is in the specified LDAP group.
     * @param User $authenticatedUser The adldap user to check.
     * @return boolean
     * @throws LdapGroupException Thrown when the user is not authorized.
     */
    protected function checkLDAPGroups(User $authenticatedUser)
    {
        $group = $this->groupFinder->find($authenticatedUser);

        if ($group !== null) {
            $this->chosenGroup = $group;

            return true;
        }

        // If the user is not in the correct group, then we need to check the
        // users table to see if they have been given explicit access. At
        // this point the user has already been authorized by ldap
        $this->checkIfLdapUserHasBeenGivenAccess($authenticatedUser);

        return true;
    }

    /**
     * Retrieve a user by the given credentials.
     * @param User $authenticatedUser User to check.
     * @return void
     * @throws LdapGroupException Thrown when the user is not explictly given access.
     */
    public function checkIfLdapUserHasBeenGivenAccess(User $authenticatedUser)
    {
        $user = $this->getEloquentUserFromLdapUser($authenticatedUser);
        $user = $this->removeUserIfNoAccess($user);

        if (is_null($user)) {
            throw new LdapGroupException('You do not have permission to access this CMS.');
        }
    }

    /**
     * Gets the eloquent user from the ldap user.
     * @param User $authenticatedUser Adldap to get Eloquent user.
     * @return Model
     */
    protected function getEloquentUserFromLdapUser(User $authenticatedUser)
    {
        $credentials = $this->getDatabaseCredentials($authenticatedUser);

        $query = $this->createModel()->newQuery();

        if (!str_contains($credentials->key, 'password')) {
            $query->where($credentials->key, $credentials->value);
        }

        return $query->first();
    }

    /**
     * Set allowed groups to check when logging in.
     * @param mixed $allowedGroups Groups to set on finder.
     * @return void
     */
    public function setAllowedGroups($allowedGroups)
    {
        $this->groupFinder->setAllowedGroups($allowedGroups);
    }

    /**
     * Get the Ldap credentials for a database lookup
     * @param User $authenticatedUser User to get credentials from.
     * @return array
     */
    protected function getDatabaseCredentials(User $authenticatedUser)
    {
        // Get the username input attributes
        $attributes = $this->getUsernameAttribute();
        // Get the input key
        $key = key($attributes);
        // Get the username from the AD model.
        $username = $authenticatedUser->{$attributes[$key]};

        // Make sure we retrieve the first username
        // result if it's an array.
        if (is_array($username)) {
            $username = array_get($username, 0);
        }

        $credentials = (object) [
            'key' => $key,
            'value' => $username
        ];

        return $credentials;
    }

    /**
     * Removes the user record if the user doesn't have explicted access.
     * @param Model $user Model to delete if it doesn't have access.
     * @return Model|null
     */
    public function removeUserIfNoAccess(Model $user = null)
    {
        if (!$user instanceof Model) {
            return null;
        }

        if ($this->userHasExplicitAccess($user)) {
            return $user;
        }

        $user->delete();
        return null;
    }

    /**
     * Remove a user that is no longer allowed to access the site.
     * @param  Model $user Model to check for explicit access.
     * @return boolean
     */
    protected function userHasExplicitAccess(Model $user)
    {
        return (boolean) $user->explicit_access;
    }
}
